import 'package:comunic/helpers/conversations_helper.dart';
import 'package:comunic/models/conversation.dart';
import 'package:comunic/ui/screens/conversation_screen.dart';
import 'package:comunic/ui/widgets/safe_state.dart';
import 'package:comunic/ui/widgets/tablet_mode/conversations/conversation_window_container.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/material.dart';

/// Single conversation window
///
/// @author Pierre Hubert

class ConversationWindow extends StatefulWidget {
  final int convID;
  final Function() onClose;

  const ConversationWindow({
    Key key,
    @required this.convID,
    @required this.onClose,
  })  : assert(convID != null),
        assert(onClose != null),
        super(key: key);

  @override
  _ConversationWindowState createState() => _ConversationWindowState();
}

class _ConversationWindowState extends SafeState<ConversationWindow> {
  Conversation _conversation;
  String _convTitle;
  bool _error = false;
  bool _collapsed = false;

  int get _convID => widget.convID;

  final _convKey = UniqueKey();

  void _setError(bool e) => setState(() => _error = e);

  void _toggleVisibility() => setState(() => _collapsed = !_collapsed);

  Future<void> _refresh() async {
    try {
      _setError(false);

      final conversation =
          await ConversationsHelper().getSingle(_convID, force: true);
      assert(conversation != null);
      final name =
          await ConversationsHelper.getConversationNameAsync(conversation);
      assert(name != null);

      setState(() {
        _conversation = conversation;
        _convTitle = name;
      });
    } catch (e, s) {
      _setError(true);
      print("Could not refresh the list of conversations! $e\n$s");
      showSimpleSnack(context, tr("Could not load conversation information!"));
    }
  }

  @override
  void initState() {
    super.initState();
    _refresh();
  }

  @override
  Widget build(BuildContext context) {
    // In case of error
    if (_error)
      return ConversationWindowContainer(
        title: Text(tr("Error")),
        onClose: widget.onClose,
        onToggleCollapse: _toggleVisibility,
        isCollapsed: _collapsed,
        body: buildErrorCard(tr("Could not load conversation information!"),
            actions: [
              FlatButton(
                textColor: Colors.white,
                onPressed: _refresh,
                child: Text(tr("Try again").toUpperCase()),
              )
            ]),
      );

    // If it is still loading
    if (_conversation == null)
      return ConversationWindowContainer(
        title: Text(tr("Loading...")),
        onClose: widget.onClose,
        onToggleCollapse: _toggleVisibility,
        isCollapsed: _collapsed,
        body: buildCenteredProgressBar(),
      );

    return ConversationWindowContainer(
      title: Text(_convTitle),
      onClose: widget.onClose,
      onToggleCollapse: _toggleVisibility,
      isCollapsed: _collapsed,
      body: ConversationScreen(
        key: _convKey,
        conversationID: _convID,
      ),
    );
  }
}
